package org.freecompany.imp;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.optional.junit.JUnitTask;

import static java.util.logging.Level.OFF;
import static java.util.logging.Level.parse;
import static java.util.logging.Logger.getLogger;
import static org.freecompany.util.multicaster.LoggerNotification.enableBuffering;

/**
 * This task extends the core task for running unit tests to enable
 * customizable logging per run. The behavior is identical to the
 * base task class except that an optional log level can be specified
 * which will be universally applied against all packages in the
 * master logger.  Existing levels for each handler and the root
 * logger are saved and reset after the tests complete.
 */
public class JUnit extends JUnitTask {

	protected Level level = OFF;
	protected String[] packages = new String[] { ""};

	/**
	 * For some unknown reason the base class constructor throws {@link Exception}
	 * so our default constructor must match.
	 */
	public JUnit() throws Exception {}

	public void execute() throws BuildException {
		log( "Running tests with log level " + level);
		final Map< Logger, Level> loggers = new HashMap< Logger, Level>();
		final Map< Handler, Level> handlers = new HashMap< Handler, Level>();

		for ( String name : packages) {
			final Logger logger = getLogger( name);
			loggers.put( logger, logger.getLevel());
			logger.setLevel( level);
		}
		for ( Handler handler : getLogger( "").getHandlers()) {
			handlers.put( handler, handler.getLevel());
			handler.setLevel( level);
		}

		super.execute();

		for ( Logger logger : loggers.keySet()) logger.setLevel( loggers.get( logger));
		for ( Handler handler : handlers.keySet()) handler.setLevel( handlers.get( handler));
	}

	public void setLoggers( final String packages) {
		this.packages = packages.split( ",");
	}

	public void setLevel( final String level) {
		this.level = parse( level);
	}
}
