package org.freecompany.imp;

import org.freecompany.brimstone.module.Module;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.logging.Logger;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

import static java.util.logging.Logger.getLogger;
import static java.util.logging.Level.FINE;
import static org.freecompany.imp.Constants.PREFIX_PATH;
import static org.freecompany.imp.Constants.PREFIX_URL;

/**
 * Locally caches a remote resource and makes it available within the build system by an Ant
 * property.  The name of the property is either the provided name attribute, or else the name
 * of the artifact the URI points to.
 */
public class Resolve extends Task {

	protected final Logger logger = getLogger( Resolve.class.getName());
	protected String name;
	protected String uri;
	protected String path;
	protected File file;
	protected URL url;

	protected File load( final URL url) throws MalformedURLException, URISyntaxException, IOException {
		logger.log( FINE, "Sending off ''{0}''.", url);
		final Object object = Module.load( url);
		logger.log( FINE, "Module resolved ''{0}''.", object);
		if ( object instanceof URL) {
			logger.log( FINE, "Loading module fragment ''{0}''.", object);
			return load(( URL) object);
		} else {
			logger.log( FINE, "Caching url ''{0}''.", url);
			return Access.lookupFile( url);
		}
	}

	/**
	 * Resolves a given URI relative to the project module URL.
	 */
	public void execute() throws BuildException {
		Util.initialize( getProject());
		if ( uri == null) throw new BuildException( "'uri' is a required attribute of the resolve task.");
		try {
			this.url = Util.loadUrl( getProject(), uri);
			this.file = load( url);
			logger.log( FINE, "Found url ''{0}'' and file ''{1}''.", new Object[] { url, file});
			this.path = file.getAbsolutePath();
		} catch ( Exception e) {
			throw new BuildException( e);
		}

		final String binding = getBinding();
		if ( binding != null) {
			Util.setProperty( getProject(), PREFIX_PATH + binding, path);
			Util.setProperty( getProject(), PREFIX_URL + binding, uri);
		}
	}

	public void setName( String name) {
		this.name = name;
	}

	public void setUri( String uri) {
		this.uri = uri;
	}

	public String getUri() {
		return uri;
	}

	public URL getUrl() {
		return url;
	}

	public File getFile() {
		return file;
	}

	public String getPath() {
		return path;
	}

	/**
	 * Defines the name attribute as being requires for this class. Subclasses which do not want
	 * to automatically bind to an Ant property can make this optional.
	 */
	protected String getBinding() {
		if ( name == null) throw new BuildException( "Name is a required attribute of the resolve task.");
		return name;
	}
}
